'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var r = require('ramda');
var invariant = _interopDefault(require('invariant'));

var paths = (function (ps, obj) {
  return r.map(function (path) {
    return r.path(path, obj);
  }, ps);
});

var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

var toPath = r.pipe(r.split(/[^\w\d]+/g), r.reject(r.isEmpty));

var edgeToTransformation = function edgeToTransformation(_ref) {
  var _ref2 = slicedToArray(_ref, 2),
      fRawPath = _ref2[0],
      depRawPaths = _ref2[1];

  var fPath = toPath(fRawPath);
  var depPaths = r.map(toPath, depRawPaths);
  var fLens = r.lensPath(fPath);

  return function (graph) {
    // Start of debug
    if (process.env.NODE_ENV !== 'production') {
      // Util functions we will need but only in non-prod
      var isPopulated = r.complement(r.isEmpty);
      var isDependencyInGraph = function isDependencyInGraph(depPath, graph) {
        return r.pipe(r.init, function (x) {
          return r.path(x)(graph);
        }, r.has(r.last(depPath)))(depPath);
      };

      // Check that the key exists for the func
      invariant(typeof r.view(fLens)(graph) === 'function', 'Function [' + fRawPath + '] not found in graph');

      // Check that the dep exists
      // It is important to note that a dep could be
      // null or undefined. Therefore we need to check the key
      // not the value
      r.forEachObjIndexed(function (depPath, idx) {
        invariant(isPopulated(depPath), 'Empty path at index [' + idx + '] for [' + fRawPath + ']');

        invariant(isDependencyInGraph(depPath, graph), 'Dependency [' + depRawPaths[idx] + '] for [' + fRawPath + '] not found in graph');
      })(depPaths);
    }
    // End of debug
    return r.over(fLens, function (f) {
      return r.partial(f, paths(depPaths, graph));
    }, graph);
  };
};

var mapEdgesToTransformations = r.map(edgeToTransformation);

var index = (function (edges, graph) {
  var transformations = mapEdgesToTransformations(edges);

  return r.pipe.apply(null, transformations)(graph);
});

module.exports = index;
