import { pipe, split, reject, isEmpty, map, lensPath, complement, init, path, has, last, view, forEachObjIndexed, over, partial } from 'ramda';
import invariant from 'invariant';

var paths = (function (ps, obj) {
  return map(function (path$$1) {
    return path(path$$1, obj);
  }, ps);
});

var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

var toPath = pipe(split(/[^\w\d]+/g), reject(isEmpty));

var edgeToTransformation = function edgeToTransformation(_ref) {
  var _ref2 = slicedToArray(_ref, 2),
      fRawPath = _ref2[0],
      depRawPaths = _ref2[1];

  var fPath = toPath(fRawPath);
  var depPaths = map(toPath, depRawPaths);
  var fLens = lensPath(fPath);

  return function (graph) {
    // Start of debug
    if (process.env.NODE_ENV !== 'production') {
      // Util functions we will need but only in non-prod
      var isPopulated = complement(isEmpty);
      var isDependencyInGraph = function isDependencyInGraph(depPath, graph) {
        return pipe(init, function (x) {
          return path(x)(graph);
        }, has(last(depPath)))(depPath);
      };

      // Check that the key exists for the func
      invariant(typeof view(fLens)(graph) === 'function', 'Function [' + fRawPath + '] not found in graph');

      // Check that the dep exists
      // It is important to note that a dep could be
      // null or undefined. Therefore we need to check the key
      // not the value
      forEachObjIndexed(function (depPath, idx) {
        invariant(isPopulated(depPath), 'Empty path at index [' + idx + '] for [' + fRawPath + ']');

        invariant(isDependencyInGraph(depPath, graph), 'Dependency [' + depRawPaths[idx] + '] for [' + fRawPath + '] not found in graph');
      })(depPaths);
    }
    // End of debug
    return over(fLens, function (f) {
      return partial(f, paths(depPaths, graph));
    }, graph);
  };
};

var mapEdgesToTransformations = map(edgeToTransformation);

var index = (function (edges, graph) {
  var transformations = mapEdgesToTransformations(edges);

  return pipe.apply(null, transformations)(graph);
});

export default index;
